@@ -216,6 +216,19 @@ class FilteredRelationTests(TestCase):
             str(queryset.query),
         )
 
+    def test_period_forbidden(self):
+        msg = (
+            "FilteredRelation doesn't support aliases with periods (got 'book.alice')."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Author.objects.annotate(
+                **{
+                    "book.alice": FilteredRelation(
+                        "book", condition=Q(book__title__iexact="poem by alice")
+                    )
+                }
+            )
+
     def test_multiple(self):
         qs = (
             Author.objects.annotate(
@@ -582,6 +595,28 @@ class FilteredRelationTests(TestCase):
             lambda x: (x.author, x.book_title, x.preferred_by_author_pk),
         )
 
+    def test_three_level_nested_chained_relations(self):
+        borrower = Borrower.objects.create(name="Jenny")
+        Reservation.objects.create(
+            borrower=borrower,
+            book=self.book1,
+            state=Reservation.STOPPED,
+        )
+        qs = Author.objects.annotate(
+            my_books=FilteredRelation("book"),
+            my_reserved_books=FilteredRelation(
+                "my_books__reservation",
+                condition=Q(my_books__reservation__state=Reservation.STOPPED),
+            ),
+            my_readers=FilteredRelation(
+                "my_reserved_books__borrower",
+                condition=Q(my_reserved_books__borrower=borrower),
+            ),
+        )
+        self.assertSequenceEqual(
+            qs.filter(my_readers=borrower).values_list("name", flat=True), ["Alice"]
+        )
+
     def test_deep_nested_foreign_key(self):
         qs = (
             Book.objects.annotate(
@@ -668,6 +703,19 @@ class FilteredRelationTests(TestCase):
                 ),
             )
 
+    def test_condition_deeper_relation_name_implicit_exact(self):
+        msg = (
+            "FilteredRelation's condition doesn't support nested relations "
+            "deeper than the relation_name (got 'book__editor__name' for 'book')."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Author.objects.annotate(
+                book_editor=FilteredRelation(
+                    "book",
+                    condition=Q(book__editor__name="b"),
+                ),
+            )
+
     def test_with_empty_relation_name_error(self):
         with self.assertRaisesMessage(ValueError, "relation_name cannot be empty."):
             FilteredRelation("", condition=Q(blank=""))
